/*
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */
/*
 * This file was generated.
 *  with google-apis-code-generator 1.3.0 (build: 2013-04-25 17:55:28 UTC)
 *  on 2013-04-25 at 18:46:42 UTC 
 */

package com.google.api.services.discovery;

import com.google.api.client.googleapis.GoogleUtils;
import com.google.api.client.googleapis.services.AbstractGoogleClientRequest;
import com.google.api.client.googleapis.services.GoogleClientRequestInitializer;
import com.google.api.client.googleapis.services.json.AbstractGoogleJsonClient;
import com.google.api.client.http.HttpRequestInitializer;
import com.google.api.client.http.HttpTransport;
import com.google.api.client.json.JsonFactory;
import com.google.api.client.json.JsonObjectParser;
import com.google.common.base.Preconditions;

/**
 * Service definition for Discovery (v1).
 *
 * <p>
 * Lets you discover information about other Google APIs, such as what APIs are available, the resource and method details for each API.
 * </p>
 *
 * <p>
 * For more information about this service, see the
 * <a href="https://developers.google.com/discovery/" target="_blank">API Documentation</a>
 * </p>
 *
 * <p>
 * This service uses {@link DiscoveryRequestInitializer} to initialize global parameters via its
 * {@link Builder}.
 * </p>
 *
 * <p>
 * Upgrade warning: this class now extends {@link AbstractGoogleJsonClient}, whereas in prior
 * version 1.8 it extended {@link com.google.api.client.googleapis.services.GoogleClient}.
 * </p>
 *
 * @since 1.3
 * @author Google, Inc.
 */
@SuppressWarnings("javadoc")
public class Discovery extends AbstractGoogleJsonClient {

  // Note: Leave this static initializer at the top of the file.
  static {
    Preconditions.checkState(GoogleUtils.VERSION.equals("1.13.2-beta"),
        "You are currently running with version %s of google-api-client. " +
        "You need version 1.13.2-beta of google-api-client to run version " +
        "1.13.2-beta of the APIs Discovery Service library.", GoogleUtils.VERSION);
  }

  /**
   * The default encoded root URL of the service. This is determined when the library is generated
   * and normally should not be changed.
   *
   * @since 1.7
   */
  public static final String DEFAULT_ROOT_URL = "https://www.googleapis.com/";

  /**
   * The default encoded service path of the service. This is determined when the library is
   * generated and normally should not be changed.
   *
   * @since 1.7
   */
  public static final String DEFAULT_SERVICE_PATH = "discovery/v1/";

  /**
   * The default encoded base URL of the service. This is determined when the library is generated
   * and normally should not be changed.
   * @deprecated (scheduled to be removed in 1.13)
   */
  @Deprecated
  public static final String DEFAULT_BASE_URL = DEFAULT_ROOT_URL + DEFAULT_SERVICE_PATH;

  /**
   * Constructor.
   *
   * <p>
   * Use {@link Builder} if you need to specify any of the optional parameters.
   * </p>
   *
   * @param transport HTTP transport
   * @param jsonFactory JSON factory
   * @param httpRequestInitializer HTTP request initializer or {@code null} for none
   * @since 1.7
   */
  public Discovery(HttpTransport transport, JsonFactory jsonFactory,
      HttpRequestInitializer httpRequestInitializer) {
    super(transport,
        jsonFactory,
        DEFAULT_ROOT_URL,
        DEFAULT_SERVICE_PATH,
        httpRequestInitializer,
        false);
  }

  /**
   * @param transport HTTP transport
   * @param httpRequestInitializer HTTP request initializer or {@code null} for none
   * @param rootUrl root URL of the service
   * @param servicePath service path
   * @param jsonObjectParser JSON object parser
   * @param googleClientRequestInitializer Google request initializer or {@code null} for none
   * @param applicationName application name to be sent in the User-Agent header of requests or
   *        {@code null} for none
   * @param suppressPatternChecks whether discovery pattern checks should be suppressed on required
   *        parameters
   */
  Discovery(HttpTransport transport,
      HttpRequestInitializer httpRequestInitializer,
      String rootUrl,
      String servicePath,
      JsonObjectParser jsonObjectParser,
      GoogleClientRequestInitializer googleClientRequestInitializer,
      String applicationName,
      boolean suppressPatternChecks) {
    super(transport,
        httpRequestInitializer,
        rootUrl,
        servicePath,
        jsonObjectParser,
        googleClientRequestInitializer,
        applicationName,
        suppressPatternChecks);
  }

  @Override
  protected void initialize(AbstractGoogleClientRequest<?> httpClientRequest) throws java.io.IOException {
    super.initialize(httpClientRequest);
  }

  /**
   * An accessor for creating requests from the Apis collection.
   *
   * <p>The typical use is:</p>
   * <pre>
   *   {@code Discovery discovery = new Discovery(...);}
   *   {@code Discovery.Apis.List request = discovery.apis().list(parameters ...)}
   * </pre>
   *
   * @return the resource collection
   */
  public Apis apis() {
    return new Apis();
  }

  /**
   * The "apis" collection of methods.
   */
  public class Apis {

    /**
     * Retrieve the description of a particular version of an api.
     *
     * Create a request for the method "apis.getRest".
     *
     * This request holds the parameters needed by the the discovery server.  After setting any optional
     * parameters, call the {@link GetRest#execute()} method to invoke the remote operation.
     *
     * @param api The name of the API.
     * @param version The version of the API.
     * @return the request
     */
    public GetRest getRest(String api, String version) throws java.io.IOException {
      GetRest result = new GetRest(api, version);
      initialize(result);
      return result;
    }

    public class GetRest extends DiscoveryRequest<com.google.api.services.discovery.model.RestDescription> {

      private static final String REST_PATH = "apis/{api}/{version}/rest";

      /**
       * Retrieve the description of a particular version of an api.
       *
       * Create a request for the method "apis.getRest".
       *
       * This request holds the parameters needed by the the discovery server.  After setting any
       * optional parameters, call the {@link GetRest#execute()} method to invoke the remote operation.
       * <p> {@link GetRest#initialize(AbstractGoogleClientRequest)} must be called to initialize this
       * instance immediately after invoking the constructor. </p>
       *
       * @param api The name of the API.
       * @param version The version of the API.
       * @since 1.13
       */
      protected GetRest(String api, String version) {
        super(Discovery.this, "GET", REST_PATH, null, com.google.api.services.discovery.model.RestDescription.class);
        this.api = Preconditions.checkNotNull(api, "Required parameter api must be specified.");
        this.version = Preconditions.checkNotNull(version, "Required parameter version must be specified.");
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public GetRest setAlt(String alt) {
        return (GetRest) super.setAlt(alt);
      }

      @Override
      public GetRest setFields(String fields) {
        return (GetRest) super.setFields(fields);
      }

      @Override
      public GetRest setKey(String key) {
        return (GetRest) super.setKey(key);
      }

      @Override
      public GetRest setOauthToken(String oauthToken) {
        return (GetRest) super.setOauthToken(oauthToken);
      }

      @Override
      public GetRest setPrettyPrint(Boolean prettyPrint) {
        return (GetRest) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public GetRest setQuotaUser(String quotaUser) {
        return (GetRest) super.setQuotaUser(quotaUser);
      }

      @Override
      public GetRest setUserIp(String userIp) {
        return (GetRest) super.setUserIp(userIp);
      }

      /** The name of the API. */
      @com.google.api.client.util.Key
      private String api;

      /** The name of the API.
       */
      public String getApi() {
        return api;
      }

      /** The name of the API. */
      public GetRest setApi(String api) {
        this.api = api;
        return this;
      }

      /** The version of the API. */
      @com.google.api.client.util.Key
      private String version;

      /** The version of the API.
       */
      public String getVersion() {
        return version;
      }

      /** The version of the API. */
      public GetRest setVersion(String version) {
        this.version = version;
        return this;
      }

    }
    /**
     * Retrieve the list of APIs supported at this endpoint.
     *
     * Create a request for the method "apis.list".
     *
     * This request holds the parameters needed by the the discovery server.  After setting any optional
     * parameters, call the {@link List#execute()} method to invoke the remote operation.
     *
     * @return the request
     */
    public List list() throws java.io.IOException {
      List result = new List();
      initialize(result);
      return result;
    }

    public class List extends DiscoveryRequest<com.google.api.services.discovery.model.DirectoryList> {

      private static final String REST_PATH = "apis";

      /**
       * Retrieve the list of APIs supported at this endpoint.
       *
       * Create a request for the method "apis.list".
       *
       * This request holds the parameters needed by the the discovery server.  After setting any
       * optional parameters, call the {@link List#execute()} method to invoke the remote operation. <p>
       * {@link List#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @since 1.13
       */
      protected List() {
        super(Discovery.this, "GET", REST_PATH, null, com.google.api.services.discovery.model.DirectoryList.class);
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public List setAlt(String alt) {
        return (List) super.setAlt(alt);
      }

      @Override
      public List setFields(String fields) {
        return (List) super.setFields(fields);
      }

      @Override
      public List setKey(String key) {
        return (List) super.setKey(key);
      }

      @Override
      public List setOauthToken(String oauthToken) {
        return (List) super.setOauthToken(oauthToken);
      }

      @Override
      public List setPrettyPrint(Boolean prettyPrint) {
        return (List) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public List setQuotaUser(String quotaUser) {
        return (List) super.setQuotaUser(quotaUser);
      }

      @Override
      public List setUserIp(String userIp) {
        return (List) super.setUserIp(userIp);
      }

      /** Only include APIs with the given name. */
      @com.google.api.client.util.Key
      private String name;

      /** Only include APIs with the given name.
       */
      public String getName() {
        return name;
      }

      /** Only include APIs with the given name. */
      public List setName(String name) {
        this.name = name;
        return this;
      }

      /** Return only the preferred version of an API. */
      @com.google.api.client.util.Key
      private Boolean preferred;

      /** Return only the preferred version of an API. [default: false]
       */
      public Boolean getPreferred() {
        return preferred;
      }

      /** Return only the preferred version of an API. */
      public List setPreferred(Boolean preferred) {
        this.preferred = preferred;
        return this;
      }

      /**
       * Convenience method that returns only {@link Boolean#TRUE} or {@link Boolean#FALSE}.
       *
       * <p>
       * Boolean properties can have four possible values:
       * {@code null}, {@link com.google.api.client.util.Data#NULL_BOOLEAN}, {@link Boolean#TRUE}
       * or {@link Boolean#FALSE}.
       * </p>
       *
       * <p>
       * This method returns {@link Boolean#TRUE} if the default of the property is {@link Boolean#TRUE}
       * and it is {@code null} or {@link com.google.api.client.util.Data#NULL_BOOLEAN}.
       * {@link Boolean#FALSE} is returned if the default of the property is {@link Boolean#FALSE} and
       * it is {@code null} or {@link com.google.api.client.util.Data#NULL_BOOLEAN}.
       * </p>
       *
       * <p>
       * Return only the preferred version of an API.
       * </p>
       */
      public boolean isPreferred() {
        if (preferred == null || preferred == com.google.api.client.util.Data.NULL_BOOLEAN) {
          return false;
        }
        return preferred;
      }

    }

  }

  /**
   * Builder for {@link Discovery}.
   *
   * <p>
   * Implementation is not thread-safe.
   * </p>
   *
   * @since 1.3.0
   */
  public static final class Builder extends AbstractGoogleJsonClient.Builder {

    /**
     * Returns an instance of a new builder.
     *
     * @param transport HTTP transport
     * @param jsonFactory JSON factory
     * @param httpRequestInitializer HTTP request initializer or {@code null} for none
     * @since 1.7
     */
    public Builder(HttpTransport transport, JsonFactory jsonFactory,
        HttpRequestInitializer httpRequestInitializer) {
      super(
          transport,
          jsonFactory,
          DEFAULT_ROOT_URL,
          DEFAULT_SERVICE_PATH,
          httpRequestInitializer,
          false);
    }

    /** Builds a new instance of {@link Discovery}. */
    @Override
    public Discovery build() {
      return new Discovery(getTransport(),
          getHttpRequestInitializer(),
          getRootUrl(),
          getServicePath(),
          getObjectParser(),
          getGoogleClientRequestInitializer(),
          getApplicationName(),
          getSuppressPatternChecks());
    }

    @Override
    public Builder setRootUrl(String rootUrl) {
      return (Builder) super.setRootUrl(rootUrl);
    }

    @Override
    public Builder setServicePath(String servicePath) {
      return (Builder) super.setServicePath(servicePath);
    }

    @Override
    public Builder setHttpRequestInitializer(HttpRequestInitializer httpRequestInitializer) {
      return (Builder) super.setHttpRequestInitializer(httpRequestInitializer);
    }

    @Override
    public Builder setApplicationName(String applicationName) {
      return (Builder) super.setApplicationName(applicationName);
    }

    @Override
    public Builder setSuppressPatternChecks(boolean suppressPatternChecks) {
      return (Builder) super.setSuppressPatternChecks(suppressPatternChecks);
    }

    /**
     * Set the {@link DiscoveryRequestInitializer}.
     *
     * @since 1.12
     */
    public Builder setDiscoveryRequestInitializer(
        DiscoveryRequestInitializer discoveryRequestInitializer) {
      return (Builder) super.setGoogleClientRequestInitializer(discoveryRequestInitializer);
    }

    @Override
    public Builder setGoogleClientRequestInitializer(
        GoogleClientRequestInitializer googleClientRequestInitializer) {
      return (Builder) super.setGoogleClientRequestInitializer(googleClientRequestInitializer);
    }
  }
}
